using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using LIT.UploadBar;

namespace InlineUploadBarWithBackgroundUpdate
{
	/// <summary>
	/// Summary description for UploadProgressBarTxn.
	/// </summary>
	public class UploadProgressBarTxn : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label lblFileName;
		protected System.Web.UI.WebControls.PlaceHolder plcAllFiles;
		protected System.Web.UI.WebControls.PlaceHolder plcProgressBar;
		protected System.Web.UI.WebControls.Label lblBytesRead;
		protected System.Web.UI.WebControls.Label lblBytesTotal;
		protected System.Web.UI.HtmlControls.HtmlInputButton btnMore;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			// When the page loads, information must be obtained
			// about the upload. The upload is identified by the
			// variable 'UploadID' passed in on the query string.

			// Obtain the unique ID from the query string.
			string uniqueId = "";
			string script = "";
			bool completed = false;

			if (Request.QueryString["UploadID"] != null)
				uniqueId = Request.QueryString["UploadID"];
			else
				completed = true;

			// Variables which will be used to read/display the results
			string
				percentage,
				sourceFileName = "",
				bytesRead,
				bytesTotal,
				timeRemaining,
				dataRate;
			int
				percentageInt = 0,
				bytesReadInt = 0,
				bytesTotalInt = 0,
				timeRemainingInt = 0;

			double dataRateDouble = 0;


			string[] fileNames = {"",""};
			// Obtain the status for this upload
			if (!completed && !UploadDetails.GetStatus(uniqueId, out sourceFileName, out bytesReadInt, out bytesTotalInt, out percentageInt, out completed, out fileNames, out timeRemainingInt, out dataRateDouble))
			{
				// Set the values to defaults.
				percentage = "0";
				bytesRead = "0";
				bytesTotal = "0";
				sourceFileName = "No file detected";
				timeRemaining = "unknown";
				dataRate = "unknown";
			}
			else
			{
				// Put the integers into the strings
				percentage = percentageInt.ToString();
				bytesRead = bytesReadInt.ToString();
				bytesTotal = bytesTotalInt.ToString();

				if (dataRateDouble > 0)
					dataRate = UploadHelperUtils.FormatBytes((long)dataRateDouble);
				else
					dataRate = "unknown";

				if (timeRemainingInt >= 0)
					timeRemaining = new TimeSpan(0, 0, timeRemainingInt).ToString();
				else
					timeRemaining = "unknown";
			}

			// Build up the client side script (javascript)
			script += "<script type=\"text/javascript\"><!--\r\n";
			script += "function SendProgressInformation()\r\n";
			script += "{\r\n \r\n";
			// Determine which panel should be displayed
			if (completed)
				script += "parent.DisplayPanel('panelForm');\r\n";
			else
			{
				script += "parent.DisplayPanel('panelProgress');\r\n";

				// Double up all slashes in the filename so that they
				// will be displayed rather than treated as escape characters.
				DoubleSlashes(ref sourceFileName);

				script += "parent.UpdateProgressBar('" + sourceFileName + "', '" + ShowFileList(fileNames) + "', '" + percentage + "', \'" + timeRemaining +  "\', '" + dataRate + "\', '" + UploadHelperUtils.FormatBytes(Int64.Parse(bytesRead)) + "', '" + UploadHelperUtils.FormatBytes(Int64.Parse(bytesTotal)) + "');\r\n";

				// Schedule a reload in 1 second.
				ScheduleReload(uniqueId, 1);
			}

			script += "}\r\n";
			script += "SendProgressInformation();\r\n";
			script += "// -->\r\n";
			script += "</script>\r\n";

			// Place the script into the page
			RegisterClientScriptBlock("progressTxn", script);
		}



		// Schedules a reload of the page
		private void ScheduleReload(string uniqueId, int reloadSeconds)
		{
			// Build the client side script used to schedule a reload
			string script = "";
			script += "<script type=\"text/javascript\"><!--\r\n";
			string path = "UploadProgressBarTxn.aspx?UploadID=" + uniqueId;
			script += "setTimeout(\"document.location.href = '" + path + "' + (toggleList ? '&ToggleList=true' : '');\"," + reloadSeconds * 1000 + ");";
			script += "// -->\r\n";
			script += "</script>\r\n";

			// Place the script into the page
			RegisterStartupScript("reload", script);
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void ToggleListDisplay()
		{
			// Toggle the display of the detailed list
			string displayList;
			if (Session["displayList"] == null)
				displayList = "false";
			else
				displayList = Session["displayList"].ToString();

			if (displayList == "true")
			{
				Session["displayList"] = "false";
			}
			else
				Session["displayList"] = "true";
		}

		private string ShowFileList(string[] fileNames)
		{
			// If the list of files contains more than 1 file, place the list of
			// files uploaded into the page, minus the last element, as this is the
			// file currently being uploaded.
			string strList = "";
			if (fileNames != null && fileNames.Length > 1)
			{
				for (int fileCount = 0; fileCount < fileNames.Length - 1; fileCount++)
				{
					string fileName = fileNames[fileCount];

					// Double up slashes for display
					DoubleSlashes(ref fileName);
					strList += fileName + "<BR>";
				}
			}
			return strList;
		}

		/// <summary>
		/// Doubles up slashes in a string so that when the string is passed to
		/// a javascript function, the slashes will be displayed rather than
		/// treated as escape characters.
		/// </summary>
		/// <param name="inputString"></param>
		private void DoubleSlashes(ref string inputString)
		{
			string outputString = "";

			for (int charCount = 0; charCount < inputString.Length; charCount++)
			{
				string currentChar = inputString.Substring(charCount, 1);
				if (currentChar == @"\")
					outputString += @"\\";
				else
					outputString += currentChar;
			}

			inputString = outputString;
		}
	}
}
